local LCA = LibCombatAlerts
local CA1 = CombatAlerts
local CA2 = CombatAlerts2

CA_Module = ZO_Object:Subclass()
local CA_Module = CA_Module


--------------------------------------------------------------------------------
-- Members that should be overridden as needed
--------------------------------------------------------------------------------

CA_Module.ID = "CA_Module"
CA_Module.NAME = "-"
CA_Module.AUTHOR = "-"
CA_Module.ZONES = { }

CA_Module.MONITOR_EFFECTS = false
CA_Module.MONITOR_UNIT_IDS = false

CA_Module.TIMER_ALERTS = { }
CA_Module.AOE_ALERTS = { }

CA_Module.DEFAULT_SETTINGS = { }

function CA_Module:PostLoad( )
end

function CA_Module:PreUnload( )
end

function CA_Module:PreStartListening( )
end

function CA_Module:PostStopListening( )
end

function CA_Module:ProcessCombatEvents( )
end

function CA_Module:OnEffectChanged( )
end

function CA_Module:GetSettingsControls( )
	return { }
end

function CA_Module:GetDebugData( )
end


--------------------------------------------------------------------------------
-- Members that probably should not be overridden
--------------------------------------------------------------------------------

CA_Module.listening = false
CA_Module.castSources = { }

function CA_Module:Load( )
	EVENT_MANAGER:RegisterForEvent(self.ID, EVENT_PLAYER_COMBAT_STATE, function(_, ...) self:OnPlayerCombatState(...) end)
	if (IsUnitInCombat("player")) then
		self:OnPlayerCombatState(true)
	end
	self:PostLoad()

	if (not CA2.sv.suppressModuleMessages) then
		CA2.ChatMessage(zo_strformat(SI_CA_MODULE_LOAD, self.NAME), true)
	end
end

function CA_Module:Unload( )
	self:PreUnload()
	self:StopListening()
	EVENT_MANAGER:UnregisterForEvent(self.ID, EVENT_PLAYER_COMBAT_STATE)

	if (not CA2.sv.suppressModuleMessages) then
		CA2.ChatMessage(zo_strformat(SI_CA_MODULE_UNLOAD, self.NAME), true)
	end
end

function CA_Module:StartListening( )
	if (not self.listening) then
		self.listening = true
		self:PreStartListening()
		EVENT_MANAGER:RegisterForEvent(self.ID, EVENT_COMBAT_EVENT, function(_, ...) self:OnCombatEvent(...) end)
		if (self.MONITOR_EFFECTS) then
			EVENT_MANAGER:RegisterForEvent(self.ID, EVENT_EFFECT_CHANGED, function(_, ...) self:OnEffectChanged(...) end)
		end
		if (self.MONITOR_UNIT_IDS) then
			LCA.ToggleUnitIdTracking(self.ID, true)
		end
		self.castSources = { }
	end
end

function CA_Module:StopListening( )
	if (self.listening) then
		self.listening = false
		EVENT_MANAGER:UnregisterForEvent(self.ID, EVENT_COMBAT_EVENT)
		EVENT_MANAGER:UnregisterForEvent(self.ID, EVENT_EFFECT_CHANGED)
		LCA.ToggleUnitIdTracking(self.ID, false)
		self:PostStopListening()
	end
end

function CA_Module:OnPlayerCombatState( inCombat )
	if (inCombat) then
		self:StartListening()
	else
		-- Avoid false positives of combat end, often caused by combat rezzes
		zo_callLater(function() if (not IsUnitInCombat("player")) then self:StopListening() end end, 3000)
	end
end

function CA_Module:OnCombatEvent( ... )
	if (self.divertNextEvent) then
		local fn = self.divertNextEvent
		self.divertNextEvent = nil
		if (fn(...)) then
			return
		end
	end

	if (self:ProcessStandardAlerts(...)) then
		self:ProcessCombatEvents(...)
	end
end

function CA_Module:ProcessStandardAlerts( result, isError, abilityName, abilityGraphic, abilityActionSlotType, sourceName, sourceType, targetName, targetType, hitValue, powerType, damageType, log, sourceUnitId, targetUnitId, abilityId, overflow )
	if (targetType == COMBAT_UNIT_TYPE_PLAYER and result == ACTION_RESULT_BEGIN and self.TIMER_ALERTS[abilityId]) then
		local options = self.TIMER_ALERTS[abilityId]
		if (LCA.isVet or not options.vet) then
			local offset = options.offset or 0
			local id = CA1.AlertCast(abilityId, sourceName, hitValue + offset, options)
			if (options[3] and sourceUnitId and sourceUnitId ~= 0) then
				self.castSources[sourceUnitId] = id
			end
		end
	elseif (CombatAlertsData.dodge.interrupts[result] and self.castSources[targetUnitId]) then
		CA1.CastAlertsStop(self.castSources[targetUnitId])
	elseif (targetType == COMBAT_UNIT_TYPE_PLAYER and CombatAlertsData.damageEvents[result] and self.AOE_ALERTS[abilityId]) then
		local params = self.AOE_ALERTS[abilityId]
		if (not (params[2] and LCA.isTank)) then
			CA2.ScreenBorderEnable(0x00990099, params[1], "CA_AOE_ALERT")
		end
	else
		return true
	end

	return false
end

function CA_Module:GetSettings( )
	if (type(self.currentSettings) ~= "table") then
		if (type(CombatAlertsModules) ~= "table") then
			CombatAlertsModules = { }
		end
		if (type(CombatAlertsModules[self.ID]) ~= "table") then
			CombatAlertsModules[self.ID] = { }
		end
		self.currentSettings = LCA.PopulateOptions(CombatAlertsModules[self.ID], self.DEFAULT_SETTINGS)
	end
	return self.currentSettings
end
