local LCA = LibCombatAlerts
local CA2 = CombatAlerts2


--------------------------------------------------------------------------------
-- Status panel
--------------------------------------------------------------------------------

do
	local Status
	local function GetStatus( )
		if (not Status) then
			Status = LCA_StatusPanel:New()
			Status:SetRepositionCallback(function(position) CA2.sv.statusPanel = position end)
		end
		return Status
	end

	function CA2.StatusEnable( options )
		if (not options) then options = { } end
		local sv = CA2.sv.statusPanel
		if (sv) then
			options.top = sv.top
			options.left = sv.left
			options.right = sv.right
			options.bottom = sv.bottom
		end
		return GetStatus():Enable(options)
	end

	function CA2.StatusDisable( )
		if (Status) then
			Status:Disable()
		end
	end

	function CA2.StatusGetOwnerId( )
		if (Status) then
			return Status:GetOwnerId()
		else
			return nil
		end
	end

	function CA2.StatusSetRowColor( ... )
		if (CA2.StatusGetOwnerId()) then
			Status:SetRowColor(...)
		end
	end

	function CA2.StatusSetRowAlpha( ... )
		if (CA2.StatusGetOwnerId()) then
			Status:SetRowAlpha(...)
		end
	end

	function CA2.StatusSetRowHidden( ... )
		if (CA2.StatusGetOwnerId()) then
			Status:SetRowHidden(...)
		end
	end

	function CA2.StatusModifyCell( ... )
		if (CA2.StatusGetOwnerId()) then
			Status:ModifyCell(...)
		end
	end

	function CA2.StatusSetCellText( r, c, text )
		if (CA2.StatusGetOwnerId()) then
			Status:ModifyCell(r, c, { text = text })
		end
	end
end


--------------------------------------------------------------------------------
-- Group status panel
--------------------------------------------------------------------------------

do
	local GroupPanel
	local function GetGroupPanel( )
		if (not GroupPanel) then
			GroupPanel = LCA_GroupPanel:New()
			GroupPanel:SetRepositionCallback(function(position) CA2.sv.groupPanel = position end)
		end
		return GroupPanel
	end

	function CA2.GroupPanelEnable( options )
		if (not options) then options = { } end
		local sv = CA2.sv.groupPanel
		if (sv) then
			options.top = sv.top
			options.left = sv.left
			options.right = sv.right
			options.bottom = sv.bottom
		end
		GetGroupPanel():Enable(options)
	end

	function CA2.GroupPanelDisable( )
		if (GroupPanel) then
			GroupPanel:Disable()
		end
	end

	function CA2.GroupPanelIsEnabled( )
		if (GroupPanel) then
			return GroupPanel:IsEnabled()
		else
			return false
		end
	end

	function CA2.GroupPanelUpdate( ... )
		if (CA2.GroupPanelIsEnabled()) then
			GroupPanel:UpdateUnitData(...)
		end
	end
end


--------------------------------------------------------------------------------
-- Screen border alerts
--------------------------------------------------------------------------------

do
	local Border
	local function GetBorder( )
		if (not Border) then
			Border = LCA_ScreenBorder:New()
		end
		return Border
	end

	function CA2.ScreenBorderEnable( color, duration, borderIdOverride )
		return GetBorder():Enable(color, duration, borderIdOverride)
	end

	function CA2.ScreenBorderDisable( borderId )
		if (Border) then
			Border:Disable(borderId)
		end
	end
end


--------------------------------------------------------------------------------
-- General
--------------------------------------------------------------------------------

function CA2.CleanupControls( )
	CA2.StatusDisable()
	CA2.GroupPanelDisable()
	CA2.ScreenBorderDisable()
end
